"""An unoffical Python wrapper for the SSI-Teledyne Next Generation class HPLC pumps."""
# make these available at package level
from py_hplc.pump import NextGenPump  # noqa: F401
from py_hplc.pump_base import NextGenPumpBase  # noqa: F401
